;;-----------------------------------------------
;; Programmer's Toolbox 
;; CADENCE May 1995 W.Kramer
;;
;;-----------------------------------------------
;; Listing 1: BALLOON_MAKE Function
;; Creates a block using (ENTMAKE)
;; 
;; Block created: BALLOON_ID consists of a circle
;; and an attribute.
;;
;; Also sets the global BALLOON_SCALE value to
;; be 1.6 times the current text size.
;;
(defun BALLOON_MAKE ()
  (entmake
    '((0 . "BLOCK")      ;;Start block define
      (2 . "BALLOON_ID") ;;name of block
      (70 . 2)           ;;attributes attached
      (10 0.0 0.0 0.0)   ;;base point
  ))
  (entmake
    '((0 . "CIRCLE")
      (8 . "0")          ;;layer name
      (10 0.0 0.0 0.0)   ;;center point
      (40 . 1.0)         ;;radius
  ))
  (entmake
   '((0 . "ATTDEF")      ;;attribute definition
     (8 . "0")           ;;layer name
     (10 -1.0 -1.0 0.0)  ;;base point direction
     (11 0.0 0.0 0.0)    ;;alignment point
     (1 . "")            ;;default value
     (2 . "BALLOON")     ;;tag name
     (3 . "Balloon ID number") ;;prompt
     (40 . 0.6)          ;;text size
     (41 . 1.0)          ;;scale
     (50 . 0.0)          ;;orientation
     (70 . 0)            ;;visibility flag
     (71 . 0)            ;;generation flag
     (72 . 4)            ;;middle justify
     (73 . 0)            ;;no vertical alignment
  ))
  (entmake
    '((0 . "ENDBLK")     ;;end of block define
  ))
  (setq BALLOON_SCALE 
     (* (getvar "TEXTSIZE") 
        1.6))
)
;;-----------------------------------------------
;; Listing 2: Balloon drawing function
;; Draws a balloon leader based on operator
;; input of points and an ID number.
;;
;; BALLOON - draws leaders with attached balloons
;; for detailing.  Balloon is a block containing
;; a single attribute which is the sequenced ID
;; number of the balloon.  Balloons for this
;; application are ALWAYS numbers.
;;
;; Uses two global variables:
;;  BALLOON_NEXT_ID  next ID number
;;  BALLOON_SCALE    insert scale
;;-----------------------------------------------
(defun C:BALLOON ( / P1   ;;point list
                     PTS  ;;list of point lists
                     TMP  ;;temporary
                     )
   ;;
   (setvar "CMDECHO" 0)
   ;;
   ;; Check to see if block defined, if not
   ;; then make the block.
   (if (null (tblsearch "BLOCK" "BALLOON_ID"))
       (BALLOON_MAKE)
   )
   ;;
   ;; Get leader starting point
   (setq P1 
     (getpoint 
        "\nBALLOON: Leader start point:"))
   (if P1 (progn
       (setq PTS (list P1))
       (while (setq P1 
                (getpoint 
                  (car PTS) 
                  " Next point: "))
          (grdraw (car PTS) P1 -1)
          (setq PTS (cons P1 PTS))
       )
       ;;
       ;; BALLOON_NEXT_ID and BALLOON_SCALE
       ;; are global variables remembered from
       ;; one run of (C:BALLOON) to the next.
       ;;
       (if (null BALLOON_NEXT_ID)
          (setq BALLOON_NEXT_ID 1)
       )
       (if (null BALLOON_SCALE)
          (setq BALLOON_SCALE 1.0)
       )
       ;;
       (setq TMP 
          (getint 
             (strcat "\nBalloon ID number <"
                     (itoa BALLOON_NEXT_ID)
                     ">: ")))
       (if TMP
          (setq BALLOON_NEXT_ID TMP))
       ;;
       ;; Build the leader object with associated
       ;; block containing attribute.
       ;;
       (if (wcmatch (getvar "ACADVER") "*12*")
           (command "_PLINE")
           (command "_LEADER")
       )
       (foreach P1 (reverse PTS)
         (command P1)
       )
       (setq P1 
          (polar 
             (car PTS) 
             (angle 
                (cadr PTS) 
                (car PTS)) 
             BALLOON_SCALE)
       )
       (if (wcmatch (getvar "ACADVER") "*12*")
          (command 
             ""  ;finish PLINE command
             "_INSERT"
             "BALLOON_ID"
             P1
             BALLOON_SCALE ;:X scale
             ""  ;;Y scale
             ""  ;;rotation
             (itoa BALLOON_NEXT_ID)
          )
          (command 
             "" ;finish leader points
             "" ;annotation options
             "_Block" ;use a block
             "BALLOON_ID"  ;block name
             P1  ;;insert point
             BALLOON_SCALE ;;X scale
             ""  ;;Y scale
             ""  ;;Rotation
             (itoa BALLOON_NEXT_ID)
          )
       )
       (setq BALLOON_NEXT_ID 
          (1+ BALLOON_NEXT_ID))
   ))
   (princ)
)
;;------------------------------------------------
;; Listing 3: Balloon resequencing function
;; Resequence ID numbers inside balloons
;;
(defun C:RESEQ ( / TMP EN EL)
   (setq TMP 
     (getint 
       (strcat 
          "\nNext balloon ID number <"
          (itoa BALLOON_NEXT_ID) ">: ")))
   (if TMP (setq BALLOON_NEXT_ID TMP))
   ;;
   (while (setq EN 
             (car 
                (entsel "\nPick a balloon: ")))
      (setq EL (entget EN))
      (if (and (= "INSERT" 
                  (cdr (assoc 0 EL)))
               (= "BALLOON_ID" 
                  (cdr (assoc 2 EL)))
          ) (progn
        ;;Retrieve and replace the attribute
        (setq EL (entget (entnext EN))
              EL (subst 
                    (cons ;;build new dotted
                       1  ;;pair (1 . id)
                       (itoa BALLOON_NEXT_ID))
                    (assoc 1 EL)
                    EL)
              BALLOON_NEXT_ID 
                 (1+ BALLOON_NEXT_ID)
        )
        (entmod EL)
        (entupd EN)
       )
       (prompt "\nMust select a balloon insert!")
      )
   )
   (princ)
)
;;----------------------------------------------- EOF
